package de.fhdw.gaming.memory;

import java.util.List;
import java.util.Optional;

import de.fhdw.gaming.core.domain.Move;
import de.fhdw.gaming.memory.impl.MemoryPairImpl;

/**
 * Organises a list of Optinal doubles, allowing to set a maximum size for is.
 */
public interface ShiftList {

    /**
     * Returns the list of all recorded outcomes as Optional<Double>.
     */
    List<MemoryPairImpl> show();

    /**
     * Determines how many outcomes can be saved in the list displyyed by the show() method.
     * If there are too many outcomes saved this will delete the oldest outcome until
     * the number of outcomes is equal to the length.
     * @param length
     */
    void setLength(int length);

    /**
     * Returns the number of outcomes that can be saves.
     */
    int getLength();
    
    /**
     * Adds an outcome as Optional<Double> to the list. If the list is already full,
     * the oldest outcome will be deleted before adding the new one.
     * @param input
     */
    void push(Optional<Double> input);

    /**
     * Adds an outcome as Optional<Double> to the list. If the list is already full,
     * the oldest outcome will be deleted before adding the new one.
     * @param lastMoveUsed
     */
    void pushMyMove(Move<?, ?> lastMoveUsed);
    
    /**
     * Adds an outcome as Optional<Double> to the list. If the list is already full,
     * the oldest outcome will be deleted before adding the new one.
     * @param lastMoveUsed
     */
    void pushOpponentMove(Move<?, ?> lastMoveUsed);

}
