package de.fhdw.gaming.memory.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Set;

import de.fhdw.gaming.core.domain.Game;
import de.fhdw.gaming.core.domain.Move;
import de.fhdw.gaming.core.domain.Observer;
import de.fhdw.gaming.core.domain.Player;
import de.fhdw.gaming.core.domain.State;
import de.fhdw.gaming.core.domain.Strategy;
import de.fhdw.gaming.memory.Key;

/**
 * An observer implementation that interacts with the memory brain to remember game outcomes.
 */
public class MemoryObserverImpl implements Observer {

    @Override
    public void started(Game<?, ?, ?, ?> game, State<?, ?> state) throws InterruptedException {
        MemoryBrainImpl.getInstance().setMemoryLength(1);
        System.out.println("Game started!");
    }

    @Override
    public void nextPlayersComputed(Game<?, ?, ?, ?> game, State<?, ?> state, Set<? extends Player<?>> players)
            throws InterruptedException {
        //System.out.println("Computed next player.");
    }

    @Override
    public void illegalPlayerRejected(Game<?, ?, ?, ?> game, State<?, ?> state, Player<?> player)
            throws InterruptedException {
        //System.out.println("Illegal player rejected!");
    }

    @Override
    public void legalMoveApplied(Game<?, ?, ?, ?> game, State<?, ?> state, Player<?> player, Move<?, ?> move)
            throws InterruptedException {
        
        //System.out.println("Legal move applied!");
        
        Player<?> opponent = null;
        for (Map.Entry<String, ?> entry : game.getPlayers().entrySet()) {
            final Player<?> possibleOpponent = (Player<?>) entry.getValue();
            if (!possibleOpponent.getName().equals(player.getName())) {
                opponent = possibleOpponent;
            }
        }
        
        final Strategy<?, ?, ?> usedStrategy = game.getStrategies().get(player.getName());
        final Strategy<?, ?, ?> opponentStrategy = game.getStrategies().get(opponent.getName());
        
        final Key currentKey1 = new MemoryKeyImpl(player, opponent, usedStrategy);
        final Key currentKey2 = new MemoryKeyImpl(opponent, player, opponentStrategy);
        
        MemoryBrainImpl.getInstance().rememberMyMove(currentKey1, move);
        MemoryBrainImpl.getInstance().rememberOpponentMove(currentKey2, move);
  
    }

    @Override
    public void illegalMoveRejected(Game<?, ?, ?, ?> game, State<?, ?> state, Player<?> player,
            Optional<Move<?, ?>> move, String reason) throws InterruptedException {
        System.out.println("Illegal move rejected!");
    }

    @Override
    public void overdueMoveRejected(Game<?, ?, ?, ?> game, State<?, ?> state, Player<?> player,
            Optional<Move<?, ?>> chosenMove) throws InterruptedException {
        //System.out.println("overdueMoveRejected!");
    }

    @Override
    public void playerResigned(Game<?, ?, ?, ?> game, State<?, ?> state, Player<?> player) throws InterruptedException {
        //System.out.println("Player resigned!");
    }

    @Override
    public void playerOvertaken(Game<?, ?, ?, ?> game, State<?, ?> state, Player<?> overtakenPlayer,
            Player<?> overtakingPlayer) throws InterruptedException {
        // Not implemented
    }

    /**
     * Every time a game finished the outcome will be remembered by the brain.
     */
    @Override
    public void finished(Game<?, ?, ?, ?> game, State<?, ?> state) throws InterruptedException {
        System.out.println("Game finished!");
        MemoryBrainImpl.getInstance().rememberOutcome(game);
        
        final Map<String, ? extends Strategy<?, ?, ?>> tempStrategies = game.getStrategies();
        final Map<String, ? extends Player<?>> tempPlayers = game.getPlayers();
        final List<String> playerNames = new ArrayList<>(tempPlayers.keySet());
        
        final Key currentKey1 = new MemoryKeyImpl(tempPlayers
                .get(playerNames.get(0)), tempPlayers.get(playerNames.get(1)), tempStrategies.get(playerNames.get(0)));
        
        final Key currentKey2 = new MemoryKeyImpl(tempPlayers
                .get(playerNames.get(1)), tempPlayers.get(playerNames.get(0)), tempStrategies.get(playerNames.get(1)));
        
//        System.out.println(MemoryBrainImpl.getInstance().getMemory(currentKey1).show());
//        System.out.println(MemoryBrainImpl.getInstance().getMemory(currentKey2).show().toString());
        
        final List<MemoryPairImpl> pairList = MemoryBrainImpl.getInstance().getMemory(currentKey1).show();
        final List<MemoryPairImpl> pairList2 = MemoryBrainImpl.getInstance().getMemory(currentKey2).show();
        
//        MemoryPairImpl testPair1 = MemoryBrainImpl.getInstance().getMemory(currentKey1).show().get(0);
//        MemoryPairImpl testPair2 = MemoryBrainImpl.getInstance().getMemory(currentKey2).show().get(0);
        
        pairList.forEach(pair -> System.out.print("[" + pair.toString() + "]"));
        System.out.println("");
        pairList2.forEach(pair -> System.out.print("[" + pair.toString() + "]"));
        System.out.println("");
        
        
    }

}
